--[[
Name: SpecialEvents-Loot-1.0
Revision: $Rev: 17337 $
Author: Tekkub Stoutwrithe (sky.shell@gmail.com)
Website: http://www.wowace.com/
Description: Special events for players looting items.
Dependencies: AceLibrary, AceEvent-2.0, Deformat
]]

local vmajor, vminor = "SpecialEvents-Loot-1.0", "$Revision: 17337 $"

if not AceLibrary then error(vmajor .. " requires AceLibrary.") end
if not AceLibrary:HasInstance("AceEvent-2.0") then error(vmajor .. " requires AceEvent-2.0.") end
if not AceLibrary:HasInstance("Deformat-2.0") then error(vmajor .. " requires Deformat-2.0.") end
if not AceLibrary:IsNewVersion(vmajor, vminor) then return end

local lib = {}
AceLibrary("AceEvent-2.0"):embed(lib)
local deformat = AceLibrary("Deformat-2.0")


----------------------------
--     Initializaiton    --
----------------------------

-- Activate a new instance of this library
function activate(self, oldLib, oldDeactivate)
	if oldLib then
		self.vars = oldLib.vars
		oldLib:UnregisterAllEvents()
	else
		self.vars = { }
	end
	self:RegisterEvent("CHAT_MSG_LOOT", "Handler")
	self:RegisterEvent("CHAT_MSG_MONEY", "Handler")
	self:RegisterEvent("SpecialEvents_LootDummy", "Handler")
	
	self.coinage = { { GOLD, 10000 }, { SILVER, 100 }, { COPPER, 1 } }
	self.unsortedloot = 0
	self.unsortedroll = 0
	self.matchcache = { }
	
	-- [1] Self count [2] Group count [3] Pattern [4 key] Recipient [5 key] Item [6 key] Count
	self.patterns = {
		{ 0, 0, YOU_LOOT_MONEY, 'self', 'coin', 1 },
		{ 0, 0, LOOT_MONEY_SPLIT, 'self', 'coin', 1 },
		{ 0, 0, LOOT_ITEM_MULTIPLE, 'self', 1, 2, 3 },
		{ 0, 0, LOOT_ITEM, 1, 2 },
		{ 0, 0, LOOT_ITEM_SELF_MULTIPLE, 'self', 1, 2 },
		{ 0, 0, LOOT_ITEM_SELF, 'self', 1 },
		{ 0, 0, LOOT_ITEM_PUSHED_SELF_MULTIPLE, 'self', 1, 2 },
		{ 0, 0, LOOT_ITEM_PUSHED_SELF, 'self', 1 },
	}
	
--	self.rollpatterns = {
--		
--	}
--	
--	self.selectpatterns = {
--		{ 0, LOOT_ROLL_GREED,'Greed' },
--		{ 0, LOOT_ROLL_NEED, 'Need' },
--		{ 0, LOOT_ROLL_PASSED, 'Pass' }
--	}

--LOOT_ROLL_GREED = "%s has selected Greed for: %s|Hitem:%d:%d:0:0:0:0:%d:%d|h[%s]|h%s";
--LOOT_ROLL_NEED = "%s has selected Need for: %s|Hitem:%d:%d:0:0:0:0:%d:%d|h[%s]|h%s";
--LOOT_ROLL_PASSED = "%s passed on: %s|Hitem:%d:%d:0:0:0:0:%d:%d|h[%s]|h%s";

	if oldDeactivate then oldDeactivate(oldLib) end
end

function lib:Handler(text)
	--DevTools_Dump(text)
	-- Handle group mode
--	if GetLootMethod() == 'group' then
--		if self.unsortedroll > 15 then
--			table.sort(self.rollpatterns, function(a, b) return a[1] < b[1] end)
--		else
--			self.unsorted = self.unsorted + 1
--		end
--		for i, v in ipairs(self.rollpatterns) do
--			local m = self.matchcache
--			m = { deformat(text, v[2]) }
--			if m[1] then
--				DevTools_Dump(m)
--				return self:TriggerEvent('SpecialEvents_RollSelect'..v[3], m[1], ("%s|Hitem:%d:%d:0:0:0:0:%d:%d|h[%s]|h"):format(m[2], m[3], m[4], m[5], m[6], m[7]), m[8])
--			end
--		end
--	end
	-- Optimize by grouped/not and weight patterns by usage
	local group = (GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0) and 2 or 1
	if not self.currentsort or self.currentsort ~= group or self.unsortedloot > 15 then
		self.unsortedloot = 0
		self.currentsort = group
		table.sort(self.patterns, function(a, b) return a[group] < b[group] end)
	else
		self.unsortedloot = self.unsortedloot + 1
	end
	for i, v in ipairs(self.patterns) do
		local matches = self.matchcache
		matches = { deformat(text, v[3]) }
	-- [1] Self count [2] Group count [3] Pattern [4 key] Recipient [5 key] Item [6 key] Count
		if matches[1] then
			v[group] = v[group] + 1
			recipient = v[4] == "self" and UnitName("player") or matches[v[4]]
			local item = v[5] == "coin" and "coin" or matches[v[5]]
			local count = item == "coin" and (self:ParseCoinString(matches[v[6]]) or 0) or (v[6] ~= nil and matches[v[6]] or 1)
			if item == "coin" then
				return self:TriggerEvent("SpecialEvents_CoinLooted", recipient, count, matches[v[6]])
			else
				return self:TriggerEvent("SpecialEvents_ItemLooted", recipient, count, item)
			end
		end
	end
end

function SEEL(text)
	lib:Handler(text)
end

function lib:ParseCoinString(tstr)
	local total, tc = 0
	for k, v in pairs(self.coinage) do
		_, _, tc = tstr:find("(%d+) "..v[1])
		if tc then
			total = total + (tc * v[2])
		end
	end
	return total
end
--
--function XLootMonitor:AddLoot(item, recipient, count, class, classname)
--	local  itemName, itemLink, itemQuality, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture
--	
-- 	if item ~= "coin" then
-- 		itemName, itemLink, itemQuality, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(item)
--		if not itemName then
--			return nil
--		end
--		self:TriggerEvent("XLoot_Item_Recieved", item, recipient, count, class, classname, itemTexture, itemName, itemLink)
--		if recipient == UnitName("player") then
--			if itemQuality < self.db.profile.selfqualitythreshold then
--				return nil
--			end
--		else
--			if itemQuality < self.db.profile.qualitythreshold then
--				return nil
--			end
--		end
--	else
--		itemName = "coin"
--		self:TriggerEvent("XLoot_Item_Recieved", item, recipient, count, class, classname, nil, itemName)
--		if not self.db.profile.money then
--			return
--		end
--	end
--end

--------------------------------
--      Load this bitch!      --
--------------------------------
AceLibrary:Register(lib, vmajor, vminor, activate)